package com.gemserk.games.discoverthename.entities;

import java.util.Collection;
import java.util.HashMap;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.opengl.SlickCallable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.entities.Entity;
import com.gemserk.componentsengine.messages.ChildrenManagementMessageFactory;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.predicates.EntityPredicates;
import com.gemserk.componentsengine.properties.FixedProperty;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.PropertiesMapBuilder;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.render.RenderQueue;
import com.gemserk.componentsengine.slick.render.SlickCallableRenderObject;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.games.discoverthename.AnimationHelper;
import com.gemserk.games.discoverthename.components.AnimationRenderComponent;
import com.gemserk.games.discoverthename.levels.BallDescription;
import com.gemserk.games.discoverthename.renderer.RenderUtils;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.inject.Inject;

@SuppressWarnings( { "unchecked", "unused" })
public class BallEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(BallEntityBuilder.class);

	@Inject
	ChildrenManagementMessageFactory childrenManagementMessageFactory;

	@Inject
	MessageQueue messageQueue;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		tags("ball");

		final BallDescription ballDescription = parameters.get("boxDescription");

		property("position", ballDescription.position.copy());
		property("size", slick.vector(1, 1));
		property("targetPosition", slick.vector(0, 0));
		property("direction", slick.vector(1, 0));
		property("desiredDirection", slick.vector(1, 0));

		float ballRadius = 16f;

		float ballPerimeter = (float) (2 * Math.PI * ballRadius);

		float frameSize = ballPerimeter / 128f;

		Resource<Animation> animationResource = resourceManager.get("ballAnimation");

		property("animation", animationResource);
		property("animationHelper", new AnimationHelper(animationResource, frameSize));

		component(new AnimationRenderComponent("renderer")).withProperties(new ComponentProperties() {
			{
				propertyRef("size");
				property("layer", 10);
				property("color", slick.color(0.7f, 0.9f, 0.7f, 1f));
				property("direction", new FixedProperty(entity) {
					@Override
					public Object get() {
						Vector2f direction = Properties.getValue(entity, "direction");
						return direction.add(-90);
					}
				});
			}
		});

		component(new ImageRenderableComponent("ballShadow")).withProperties(new ComponentProperties() {
			{
				propertyRef("size");
				property("image", resourceManager.get("ballshadow"));
				property("color", slick.color(1f, 1f, 1f, 1f));
				property("direction", slick.vector(1f, 0f));
				property("layer", 9);
				property("position", new FixedProperty(entity) {

					Vector2f offset = slick.vector(5, 5);

					@Override
					public Object get() {
						Vector2f position = Properties.getValue(entity, "position");
						return position.copy().add(offset);
					}
				});
			}
		});

		component(new ReferencePropertyComponent("renderDesiredDirection") {

			@EntityProperty
			Property<Vector2f> position;

			@EntityProperty
			Property<Boolean> shouldMove;

			@EntityProperty
			Property<Vector2f> targetPosition;

			@Handles
			public void render(Message message) {

				if (!shouldMove.get())
					return;

				RenderQueue renderQueue = Properties.getValue(message, "renderer");

				int layer = 100;

				renderQueue.enqueue(new SlickCallableRenderObject(layer) {

					@Override
					public void execute(Graphics graphics) {
						SlickCallable.enterSafeBlock();
						RenderUtils.renderArrow(position.get(), targetPosition.get(), 2f, 14f, slick.color(0.7f, 0.7f, 0.7f, 0.3f), slick.color(1f, 0.6f, 0f, 1f));
						SlickCallable.leaveSafeBlock();
					}
				});

			}

		});

		property("force", slick.vector(0, 0));
		property("velocity", slick.vector(0, 0));
		property("maxVelocity", parameters.get("maxVelocity"));
		property("mass", 1f);

		component(new ReferencePropertyComponent("movement") {

			@EntityProperty
			Property<Vector2f> position;

			@EntityProperty
			Property<Vector2f> velocity;

			@EntityProperty
			Property<Vector2f> force;

			@EntityProperty
			Property<Float> mass;

			@EntityProperty
			Property<AnimationHelper> animationHelper;

			float pixelsPerMeter = 10f;
			
			@EntityProperty
			Property<Float> maxVelocity;

			@Handles
			public void update(Message message) {

				Integer delta = Properties.getValue(message, "delta");
				float deltaFloat = 0.001f * (float) delta;

				Vector2f acceleration = force.get().copy().scale(1f / mass.get());

				Vector2f newVelocity = velocity.get().copy().add(acceleration.copy().scale(deltaFloat * pixelsPerMeter));
				Vector2f newPosition = position.get().copy().add(velocity.get().copy().scale(deltaFloat * pixelsPerMeter));
				
				float length = newVelocity.length();
				if (length > maxVelocity.get()) 
					newVelocity.scale(maxVelocity.get() / length);
				
				animationHelper.get().forward(newPosition.distance(position.get()));

				force.get().set(0, 0);

				velocity.get().set(newVelocity);
				position.get().set(newPosition);

			}

		});

		property("shouldMove", true);
		property("shouldMoveDistance", 80f);

		component(new ReferencePropertyComponent("updateDesiredDirection") {

			@Inject
			Input input;

			@EntityProperty
			Property<Vector2f> direction;

			@EntityProperty
			Property<Vector2f> desiredDirection;

			@EntityProperty
			Property<Vector2f> position;

			@EntityProperty
			Property<Vector2f> targetPosition;

			@EntityProperty
			Property<Boolean> shouldMove;

			@EntityProperty
			Property<Float> shouldMoveDistance;

			@EntityProperty
			Property<Vector2f> force;
			
			float minForce;
			
			float maxForce;

			@Handles
			public void update(Message message) {
				targetPosition.get().set((float) input.getMouseX(), (float) input.getMouseY());
				Vector2f sub = targetPosition.get().copy().sub(position.get());
				float length = sub.length();
				
				force.set(sub.copy().scale(0.01f));
				
				desiredDirection.set(sub.normalise());

				direction.set(desiredDirection.get().copy());
//				force.set(desiredDirection.get().copy().scale(0.1f));
			}

		});

		component(new ReferencePropertyComponent("blockOverDetector") {

			@EntityProperty
			Property<Vector2f> position;
			
			@Handles
			public void update(Message message) {
				
				final Collection<Entity> nearBlocks = entity.getRoot().getEntities(Predicates.and(EntityPredicates.withAllTags("block"), new Predicate<Entity>() {
					@Override
					public boolean apply(Entity block) {
						Rectangle bounds = Properties.getValue(block, "bounds");
						Vector2f blockPosition = Properties.getValue(block, "position");
						return bounds.contains(position.get().x - blockPosition.x, position.get().y - blockPosition.y);
					}
				}));

				if (nearBlocks.isEmpty()) {
					// ball is in the air!!
					// remove the ball, add new ball animation or something...
					
					Entity fallingBall = templateProvider.getTemplate("entities.fallingBall").instantiate(entity.getId(), new HashMap<String, Object>(){{
						put("position", ((Vector2f)Properties.getValue(entity, "position")).copy());
						put("direction", ((Vector2f)Properties.getValue(entity, "direction")).copy());
						put("velocity", ((Vector2f)Properties.getValue(entity, "velocity")).copy());
						put("animation", Properties.getValue(entity, "animation"));
						put("animationHelper", Properties.getValue(entity, "animationHelper"));
					}});
					
					Entity parent = entity.getParent();
					messageQueue.enqueue(childrenManagementMessageFactory.removeEntity(entity));
					messageQueue.enqueue(childrenManagementMessageFactory.addEntity(fallingBall, parent));
					
					System.out.println("ball in the air!!");
					
					return;
				}

				messageQueue.enqueue(new Message("onBallOver", new PropertiesMapBuilder() {
					{
						property("ball", entity);
						property("blocks", nearBlocks);
					}
				}.build()));

			}

		});

	}
}
