package com.gemserk.games.discoverthename.entities;

import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.messages.ChildrenManagementMessageFactory;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.properties.FixedProperty;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.games.discoverthename.AnimationHelper;
import com.gemserk.games.discoverthename.components.AnimationRenderComponent;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;

@SuppressWarnings( { "unchecked", "unused" })
public class FallingBallEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(FallingBallEntityBuilder.class);

	@Inject
	ChildrenManagementMessageFactory childrenManagementMessageFactory;

	@Inject
	MessageQueue messageQueue;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		property("position", parameters.get("position"));
		property("direction", parameters.get("direction"));
//		property("velocity", parameters.get("velocity"));
		property("velocity", slick.vector(0, 0));
		property("size", slick.vector(1, 1));
		property("animation", parameters.get("animation"));
		property("animationHelper", parameters.get("animationHelper"));
		
		property("sound", resourceManager.get("fall"));
		
		Resource<Sound> sound = Properties.getValue(entity, "sound");
		
		sound.get().play();

		component(new AnimationRenderComponent("renderer")).withProperties(new ComponentProperties() {
			{
				propertyRef("size");
				propertyRef("direction");
				property("layer", 0);
				property("color", slick.color(0.7f, 0.9f, 0.7f, 1f));
				property("direction", new FixedProperty(entity) {
					@Override
					public Object get() {
						Vector2f direction = Properties.getValue(entity, "direction");
						return direction.copy().add(-90);
					}
				});
			}
		});

		component(new ImageRenderableComponent("ballShadow")).withProperties(new ComponentProperties() {
			{
				propertyRef("size");
				property("image", resourceManager.get("ballshadow"));
				property("color", slick.color(1f, 1f, 1f, 1f));
				property("direction", slick.vector(1f, 0f));
				property("layer", -1);
				property("position", new FixedProperty(entity) {

					Vector2f offset = slick.vector(5, 5);

					@Override
					public Object get() {
						Vector2f position = Properties.getValue(entity, "position");
						Vector2f size = Properties.getValue(entity, "size");
						return position.copy().add(offset.copy().scale(size.length()));
					}
				});
			}
		});

		property("force", slick.vector(0, 0));
		property("mass", 1f);

		component(new ReferencePropertyComponent("movement") {

			@EntityProperty
			Property<Vector2f> position;

			@EntityProperty
			Property<Vector2f> velocity;

			@EntityProperty
			Property<Vector2f> force;

			@EntityProperty
			Property<Float> mass;

			@EntityProperty
			Property<AnimationHelper> animationHelper;

			float pixelsPerMeter = 10f;

			@Handles
			public void update(Message message) {

				Integer delta = Properties.getValue(message, "delta");
				float deltaFloat = 0.001f * (float) delta;

				Vector2f acceleration = force.get().copy().scale(1f / mass.get());

				Vector2f newVelocity = velocity.get().copy().add(acceleration.copy().scale(deltaFloat * pixelsPerMeter));
				Vector2f newPosition = position.get().copy().add(velocity.get().copy().scale(deltaFloat * pixelsPerMeter));

				animationHelper.get().forward(newPosition.distance(position.get()));

				force.get().set(0, 0);

				velocity.get().set(newVelocity);
				position.get().set(newPosition);

			}

		});

		component(new ReferencePropertyComponent("falling") {

			@EntityProperty
			Property<Vector2f> size;

			@Handles
			public void update(Message message) {

				if (size.get().length() < 0.1f) {
					messageQueue.enqueue(childrenManagementMessageFactory.removeEntity(entity));
					return;
				}
				
				Integer delta = Properties.getValue(message, "delta");
				float deltaFloat = 0.001f * (float) delta;
				
				size.get().x -= 0.8f * deltaFloat;
				size.get().y -= 0.8f * deltaFloat;

			}

		});

	}
}
