package com.gemserk.games.discoverthename.entities;

import java.util.Collection;

import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Vector2f;

import com.gemserk.commons.animation.properties.TimeProvider;
import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.entities.Entity;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.predicates.EntityPredicates;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.inject.Inject;

@SuppressWarnings( { "unchecked" })
public class TargetEntityBuilder extends EntityBuilder {
	
	class DelayedProperty<T> implements Property<T> {
		
		private final Property<T> property;
		
		private final int delay;
		
		private final TimeProvider timeProvider;
		
		private T delayedValue;

		private long timeWhenSet = 0;

		public DelayedProperty(Property<T> property, int delay, TimeProvider timeProvider) {
			this.property = property;
			this.delay = delay;
			this.timeProvider = timeProvider;
		}

		@Override
		public T get() {
			if (timeProvider.getTime() - timeWhenSet > delay)
				return property.get();
			return delayedValue;
		}

		@Override
		public void set(T value) {
			delayedValue = property.get();
			property.set(value);
			timeWhenSet = timeProvider.getTime();
		}
		
	}

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		tags("target");

		property("position", parameters.get("position"));
		property("bounds", parameters.get("bounds"));
		property("size", slick.vector(1, 1));
		property("direction", slick.vector(1, 0));
		
		property("visited", false);

		component(new ImageRenderableComponent("renderer")).withProperties(new ComponentProperties() {
			{
				propertyRef("size");
				propertyRef("position");
				property("image", resourceManager.get("target"));
				property("color", slick.color(1f, 1f, 1f, 1f));
				property("direction", slick.vector(1f, 0f));
				property("layer", 5);
			}
		});
		
		// property("visited", new DelayedProperty<Boolean>(new SimpleProperty<Boolean>(false), 1000, new InterpolatedPropertyTimeProvider()));
		
		component(new ReferencePropertyComponent("visitWhenBallOver") {
			
			@EntityProperty
			Property<Rectangle> bounds;
			
			@EntityProperty
			Property<Vector2f> position;
			
			@EntityProperty
			Property<Boolean> visited;
			
			@EntityProperty
			Property<Resource<Sound>> sound;

			@Handles
			public void update(Message message) {
				
				if (visited.get())
					return;
				
				final Collection<Entity> balls = entity.getRoot().getEntities(Predicates.and(EntityPredicates.withAllTags("ball"), new Predicate<Entity>() {
					@Override
					public boolean apply(Entity ball) {
						Vector2f ballPosition = Properties.getValue(ball, "position");
						return bounds.get().contains(position.get().x - ballPosition.x, position.get().y - ballPosition.y);
					}
				}));
				
				if (balls.size() == 0)
					return;
				
				visited.set(true);

				sound.get().get().play();
			}

		}).withProperties(new ComponentProperties() {
			{
				property("sound", resourceManager.get("targetReached"));
			}
		});

	}
}
