package com.gemserk.games.discoverthename.entities;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.newdawn.slick.geom.Rectangle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.entities.Entity;
import com.gemserk.componentsengine.game.GlobalProperties;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.predicates.EntityPredicates;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.PropertiesMapBuilder;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.games.discoverthename.levels.BallDescription;
import com.gemserk.games.discoverthename.levels.BlockDescription;
import com.gemserk.games.discoverthename.levels.LevelDescription;
import com.gemserk.games.discoverthename.levels.TargetDescription;
import com.gemserk.resources.ResourceManager;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class WorldEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(WorldEntityBuilder.class);

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	MessageQueue messageQueue;

	@Inject
	GlobalProperties globalProperties;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		tags("world");

		final Rectangle screenBounds = (Rectangle) parameters.get("screenBounds");

		property("bounds", screenBounds);
		property("points", parameters.get("points"));
		property("level", parameters.get("level"));

		final LevelDescription level = Properties.getValue(entity, "level");

		property("backgroundImage", resourceManager.get(level.background));

		component(new ImageRenderableComponent("background")).withProperties(new ComponentProperties() {
			{
				property("color", slick.color(1, 1, 1, 1));
				property("position", slick.vector(screenBounds.getCenterX(), screenBounds.getCenterY()));
				property("direction", slick.vector(1, 0));
				property("layer", -1000);
				propertyRef("image", "backgroundImage");
			}
		});

		List<BallDescription> ballDescriptions = level.ballDescriptions;
		for (int i = 0; i < ballDescriptions.size(); i++) {
			final BallDescription ballDescription = ballDescriptions.get(i);
			child(templateProvider.getTemplate("entities.ball").instantiate("box_" + i, new HashMap<String, Object>() {
				{
					put("boxDescription", ballDescription);
					put("maxVelocity", 8f);
				}
			}));
		}

		List<BlockDescription> blockDescriptions = level.blockDescriptions;
		for (int i = 0; i < blockDescriptions.size(); i++) {
			final BlockDescription blockDescription = blockDescriptions.get(i);
			String templateName = blockDescription.breakable ? "entities.breakableBlock" : "entities.unbreakableBlock";
			child(templateProvider.getTemplate(templateName).instantiate("block_" + i, new HashMap<String, Object>() {
				{
					put("position", blockDescription.position);
					put("bounds", slick.rectangle(-24.5f, -24.5f, 49f, 49f));
				}
			}));
		}

		List<TargetDescription> targetDescriptions = level.targetDescriptions;
		for (int i = 0; i < targetDescriptions.size(); i++) {
			final TargetDescription targetDescription = targetDescriptions.get(i);
			child(templateProvider.getTemplate("entities.target").instantiate("target_" + i, new HashMap<String, Object>() {
				{
					put("position", targetDescription.position);
					put("bounds", slick.rectangle(-15f, -15f, 30f, 30f));
				}
			}));
		}

		property("gameOver", false);

		component(new ReferencePropertyComponent("logic") {

			@EntityProperty
			Property<Boolean> gameOver;

			@Handles
			public void update(Message message) {

				if (gameOver.get())
					return;

				final Collection<Entity> balls = entity.getRoot().getEntities(Predicates.and(EntityPredicates.withAllTags("ball")));

				if (balls.size() > 0) {

					// if not more targets not visited... then start win timer...

					final Collection<Entity> targets = entity.getRoot().getEntities(Predicates.and(EntityPredicates.withAllTags("target"), new Predicate<Entity>() {
						@Override
						public boolean apply(Entity target) {
							Boolean visited = Properties.getValue(target, "visited");
							return !visited.booleanValue();
						}
					}));

					if (targets.size() != 0)
						return;

					gameOver.set(true);
					messageQueue.enqueue(new Message("levelFinished", new PropertiesMapBuilder() {
						{
							property("win", true);
						}
					}.build()));

				} else {
					
					gameOver.set(true);
					messageQueue.enqueue(new Message("levelFinished", new PropertiesMapBuilder() {
						{
							property("win", false);
						}
					}.build()));
					
				}

			}

		});

	}
}
