package com.gemserk.games.discoverthename.gamestates;

import java.text.MessageFormat;
import java.util.ArrayList;

import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Vector2f;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.game.GlobalProperties;
import com.gemserk.componentsengine.input.InputMappingBuilder;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.input.KeyboardMappingBuilder;
import com.gemserk.componentsengine.input.MouseMappingBuilder;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.render.RenderQueue;
import com.gemserk.componentsengine.slick.render.SlickCallableRenderObject;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.games.discoverthename.levels.BlockDescription;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class EditorGameStateEntityBuilder extends EntityBuilder {

	private static final Logger logger = LoggerFactory.getLogger(EditorGameStateEntityBuilder.class);

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	SlickUtils slick;

	@Inject
	GlobalProperties globalProperties;

	@Inject
	ResourceManager resourceManager;

	@Inject
	MessageQueue messageQueue;

	@Override
	public void build() {

		property("screenBounds", parameters.get("screenBounds"));

		final Rectangle screenResolution = Properties.getValue(entity, "screenBounds");

		property("startPosition", slick.vector(0, 0));
		property("blockDescriptions", new BlockDescription[18][18]);
		property("endPositions", new ArrayList<Vector2f>());

		component(new ReferencePropertyComponent("printLevelComponent") {

			@EntityProperty
			Property<BlockDescription[][]> blockDescriptions;

			@EntityProperty
			Property<Vector2f> startPosition;

			@EntityProperty
			Property<ArrayList<Vector2f>> endPositions;

			@Handles
			public void printLevel(Message message) {

				System.out.println(MessageFormat.format("ball(new BallDescription()'{{'\n" + //
						"\tposition = new Vector2f( {0}f , {1}f );\n" + //
						"'}}');", startPosition.get().x, startPosition.get().y));

				ArrayList<Vector2f> endPositionList = endPositions.get();

				for (int i = 0; i < endPositionList.size(); i++) {

					Vector2f endPosition = endPositionList.get(i);

					System.out.println(MessageFormat.format("target(new TargetDescription()'{{'\n" + //
							"\tposition = new Vector2f( {0}f , {1}f );\n" + //
							"'}}');", endPosition.x, endPosition.y));

				}

				for (int i = 0; i < blockDescriptions.get().length; i++) {
					for (int j = 0; j < blockDescriptions.get()[i].length; j++) {
						BlockDescription blockDescription = blockDescriptions.get()[i][j];

						if (blockDescription == null)
							continue;

						System.out.println(MessageFormat.format("block(new BlockDescription()'{{'\n" + //
								"\tposition = new Vector2f( {0}f , {1}f );\n" + //
								"\tbreakable = {2};\n" + //
								"'}}');", blockDescription.position.x, blockDescription.position.y, blockDescription.breakable));
					}
				}

			}

			@Inject
			Input input;

			Integer selection = 1;

			@Handles
			public void startPositionSelected(Message message) {
				selection = 1;
			}

			@Handles
			public void breakableBlockSelected(Message message) {
				selection = 2;
			}

			@Handles
			public void unbreakableBlockSelected(Message message) {
				selection = 3;
			}

			@Handles
			public void endPositionSelected(Message message) {
				selection = 4;
			}

			@Handles
			public void leftmouse(Message message) {

				int mouseX = input.getMouseX();
				int mouseY = input.getMouseY();

				final int xMod = mouseX / 48;
				final int yMod = mouseY / 48;

				if (selection == 1) {
					startPosition.get().set(48 * xMod + 24, 48 * yMod + 24);
				} else if (selection == 2) {
					blockDescriptions.get()[xMod][yMod] = new BlockDescription() {
						{
							breakable = true;
							position = new Vector2f(48 * xMod + 24, 48 * yMod + 24);
						}
					};
				} else if (selection == 3) {
					blockDescriptions.get()[xMod][yMod] = new BlockDescription() {
						{
							breakable = false;
							position = new Vector2f(48 * xMod + 24, 48 * yMod + 24);
						}
					};
				} else if (selection == 4) {
					endPositions.get().add(new Vector2f(48 * xMod + 24, 48 * yMod + 24));
				}

			}

			@Handles
			public void rightmouse(Message message) {

				int mouseX = input.getMouseX();
				int mouseY = input.getMouseY();

				final int xMod = mouseX / 48;
				final int yMod = mouseY / 48;

				blockDescriptions.get()[xMod][yMod] = null;
				
				Vector2f mousePosition = new Vector2f(48 * xMod + 24, 48 * yMod + 24);
				
				if (startPosition.get().distance(mousePosition) < 1f)
					startPosition.get().set(0,0);
				
				ArrayList toRemove = new ArrayList();
				for (Vector2f endPosition : endPositions.get()) {
					if (endPosition.distance(mousePosition)< 1f)
						toRemove.add(endPosition);
				}
				endPositions.get().removeAll(toRemove);

			}

		});

		component(new ReferencePropertyComponent("renderLevelComponent") {

			@EntityProperty
			Property<BlockDescription[][]> blockDescriptions;

			@EntityProperty
			Property<Vector2f> startPosition;
			
			@EntityProperty
			Property<ArrayList<Vector2f>> endPositions;

			@Handles
			public void render(Message message) {

				RenderQueue renderQueue = Properties.getValue(message, "renderer");

				renderQueue.enqueue(new SlickCallableRenderObject(0) {

					@Override
					public void execute(Graphics graphics) {

						graphics.setColor(Color.white);

						for (int i = 0; i < 800 / 48; i++)
							graphics.drawLine(i * 48, 0, i * 48, 600);

						for (int i = 0; i < 600 / 48; i++)
							graphics.drawLine(0, i * 48, 800, i * 48);

					}
				});

				renderQueue.enqueue(new SlickCallableRenderObject(1) {

					@Override
					public void execute(Graphics graphics) {

						for (int i = 0; i < blockDescriptions.get().length; i++) {
							for (int j = 0; j < blockDescriptions.get()[i].length; j++) {
								BlockDescription blockDescription = blockDescriptions.get()[i][j];

								if (blockDescription == null)
									continue;

								if (blockDescription.breakable)
									graphics.setColor(Color.gray);
								else
									graphics.setColor(Color.white);

								graphics.pushTransform();
								graphics.translate(blockDescription.position.x, blockDescription.position.y);
								graphics.fillRect(-24f, -24f, 48f, 48f);
								graphics.popTransform();
							}
						}

						graphics.setColor(Color.red);
						graphics.pushTransform();
						graphics.translate(startPosition.get().x, startPosition.get().y);
						graphics.fillOval(-2f, -2f, 4f, 4f);
						graphics.popTransform();
						
						for (int i = 0; i < endPositions.get().size(); i++) {
							Vector2f endPosition = endPositions.get().get(i);
							graphics.setColor(Color.green);
							graphics.pushTransform();
							graphics.translate(endPosition.x, endPosition.y);
							graphics.fillOval(-2f, -2f, 4f, 4f);
							graphics.popTransform();
						}

					}
				});

			}

		});

		component(new ReferencePropertyComponent("returnToGame") {

			@Handles
			public void returnToGame(Message message) {
				messageQueue.enqueue(new Message("resume"));
				messageQueue.enqueue(new Message("restartLevel"));
			}

		});

		component(inputMappingConfiguratorProvider.get().configure(new InputMappingBuilder("inputMappingComponent") {

			@Override
			public void build() {

				keyboard(new KeyboardMappingBuilder() {
					@Override
					public void build() {
						press("1", "startPositionSelected");
						press("2", "breakableBlockSelected");
						press("3", "unbreakableBlockSelected");
						press("4", "endPositionSelected");
						press("escape", "returnToGame");
						press("space", "printLevel");
					}
				});

				mouse(new MouseMappingBuilder() {
					@Override
					public void build() {
						press("left", "leftmouse");
						press("right", "rightmouse");
						move("mouseMoved");
					}
				});
			}

		}));

	}
}
